"""sources module provides URL extraction utilities for images from the WWW."""
from bs4 import BeautifulSoup
import requests
import praw


class Reddit(object):
    """Reddit can fetch image urls from your favorite subreddit."""

    # pylint: disable=too-few-public-methods

    def __init__(
            self,
            subreddit="pics",
            valid_hosts=None,
            max_items=20):
        if valid_hosts is None:
            valid_hosts = [
                "i.imgur.com/",
                "i.reddituploads.com/",
                "i.redd.it/"]

        self.subreddit = subreddit
        self.valid_hosts = valid_hosts
        self.max_items = max_items
        self.client = praw.Reddit(user_agent="opendank")

    def fetch_images(self):
        """Fetch images from subreddit."""
        images = []

        hot_submissions = self.client.get_subreddit(
            self.subreddit).get_hot(limit=self.max_items)
        for submission in hot_submissions:
            if any(host in submission.url for host in self.valid_hosts):
                images.append(submission.url)
        return images


class HtmlSource(object):
    """HtmlSource can fetch a generic html request. Use this as a template for
    your own sources."""

    # pylint: disable=no-self-use

    def get_soup(self, url):
        """Get a bowl full of HTML soup."""
        response = requests.get(url)
        soup = BeautifulSoup(response.text, "html.parser")
        return soup

    def fetch_images(self):
        """Fetch image URLs from the HTML source."""
        raise Exception("This is a template class, you should not use this method.")


class CustomSource(HtmlSource):
    """CustomSource parses a HTML source based on a descriptive selection string."""

    def fetch_source(self, selection, tokens):
        """Uses the tokens to find something valuable in the selection."""
        results = []
        if tokens[0] == ">":
            for element in selection.find_all(tokens[1]):
                results += self.fetch_source(element, tokens[2:])
        elif tokens[0] == ":":
            attribute = tokens[1]
            tokens = tokens[3:]
            valid_qualifiers = []

            while tokens[0] not in [">", "!"]:
                valid_qualifiers.append(tokens[0])
                tokens = tokens[1:]

            for qualifier in valid_qualifiers:
                for element in selection.find_all(
                        attrs={attribute: qualifier}):
                    results += self.fetch_source(element, tokens)

        elif tokens[0] == "!":
            value = selection[tokens[1]]
            tokens = tokens[2:]
            while len(tokens) > 0:
                if tokens[0] == "prefix":
                    value = tokens[1] + value
                elif tokens[0] == "suffix":
                    value = value + tokens[1]
                tokens = tokens[2:]
            results += [value]
        return results

    def __init__(self, pattern):
        self.tokens = pattern.split(" ")

    def fetch_images(self):
        soup = self.get_soup(self.tokens[0])
        return self.fetch_source(soup, self.tokens[1:])



"""Module diashow provides functionality for displaying images from the WWW."""
import sys
import os
import glob
import datetime
import Tkinter as tk
import requests
from PIL import ImageTk, Image


class Diashow(object):
    """Class Diashow downloads and displays images."""

    # pylint: disable=too-many-instance-attributes

    def __init__(
            self,
            image_prefix="image",
            update_interval=10000,
            quit_key="<Escape>"):
        self.quit_key = quit_key
        self.update_interval = update_interval
        self.active = 0
        self.sources = []
        self.image_prefix = image_prefix
        self.last_fetch_date = datetime.date.fromtimestamp(0)
        self.images = []
        self.store_mode = "wb"

        self.window = tk.Tk()
        self.window.attributes("-fullscreen", True)
        self.window.title("opendank")
        self.window.update()
        self.width = self.window.winfo_width()
        self.height = self.window.winfo_height()
        self.panel = tk.Label(self.window)
        self.panel.pack(side="bottom", fill="both", expand="yes")

    def store_image(self, url, filename):
        """Downloads an image from a specific url and stores it."""
        response = requests.get(url)
        if response.status_code == 200:
            with open(filename, self.store_mode) as image:
                for chunk in response.iter_content(2**16):
                    image.write(chunk)
            return True
        return False

    def add_source(self, source_class):
        """Add image source to diashow."""
        self.sources.append(source_class)

    def fetch_images(self):
        """Fetch images from sources."""
        self.images = []
        image_urls = []
        for source in self.sources:
            image_urls += source.fetch_images()
        current_image = 0

        sys.stdout.write("Fetching images: [")

        for url in image_urls:
            image_path = self.image_prefix + str(current_image)
            sys.stdout.write("-")
            sys.stdout.flush()
            if self.store_image(url, image_path):
                self.images.append(image_path)
                current_image += 1
        print "]"

    def clear_cache(self):
        """Remove stored image artifacts."""
        for filename in glob.glob(self.image_prefix + "*"):
            os.remove(filename)

    def display_active(self):
        """Update screen to show active image."""
        photo = Image.open(self.images[self.active])
        target_size = (self.width, self.height)
        photo.thumbnail(target_size, Image.ANTIALIAS)
        image = ImageTk.PhotoImage(photo)
        self.panel.configure(image=image)
        self.panel.image = image

    def update(self):
        """Update the screen, images and sources."""
        current_date = datetime.date.today()
        if current_date.day != self.last_fetch_date.day:
            self.clear_cache()
            self.fetch_images()
            self.last_fetch_date = current_date
        self.active = (self.active + 1) % len(self.images)
        self.display_active()
        self.window.after(self.update_interval, self.update)

    def destroy(self, _):
        """Destroy the diashow window and free all resources."""
        self.window.destroy()
        self.clear_cache()
        sys.exit(0)

    def start(self):
        """Start the diashow."""
        self.update()
        self.window.bind(self.quit_key, self.destroy)
        self.window.mainloop()



from setuptools import setup

setup(
    name="opendank",
    version="0.1",
    description="View images from the WWW in a diashow.",
    url="https://github.com/lnsp/opendank",
    download_url="https://github.com/lnsp/opendank/tarball/0.1",
    author="the opendank community",
    author_email="lennart@mooxmirror.io",
    license="MIT",
    packages=["opendank"],
    keywords=["memes", "www", "diashow"],
    classifiers=[],
    install_requires=[
        "praw",
        "requests",
        "pillow",
        "bs4",
    ],
    scripts=["bin/opendank"],
    zip_safe=False)
